@@ -20,7 +20,6 @@
 #include "disjunct-utils.h"
 #include "error.h"
 #include "preparation.h"
-#include "read-regex.h"
 #include "regex-morph.h"
 #include "resources.h"
 #include "sat-solver/sat-encoder.h"
@@ -108,6 +107,7 @@ Parse_Options parse_options_create(void)
 	po->all_short = FALSE;
 	po->twopass_length = 30;
 	po->max_sentence_length = MAX_SENTENCE-3;
+	po->repeatable_rand = TRUE;
 	po->resources = resources_create();
 	po->display_short = TRUE;
 	po->display_word_subscripts = TRUE;
@@ -282,6 +282,14 @@ int parse_options_get_all_short_connectors(Parse_Options opts) {
 	return opts->all_short;
 }
 
+void parse_options_set_repeatable_rand(Parse_Options opts, int val) {
+	opts->repeatable_rand = val;
+}
+
+int parse_options_get_repeatable_rand(Parse_Options opts) {
+	return opts->repeatable_rand;
+}
+
 void parse_options_set_max_parse_time(Parse_Options opts, int dummy) {
 	opts->resources->max_parse_time = dummy;
 }
@@ -607,7 +615,9 @@ static void post_process_linkages(Sentence sent, Parse_Options opts)
 	}
 	else
 	{
-		sent->rand_state = N_linkages_found + sent->length;
+		if (opts->repeatable_rand)
+			sent->rand_state = N_linkages_found + sent->length;
+
 		for (in=0; in<N_linkages_alloced; in++)
 		{
 			double frac = (double) N_linkages_found;
@@ -751,6 +761,9 @@ static void post_process_linkages(Sentence sent, Parse_Options opts)
 *
 ****************************************************************/
 
+/* Its OK if this is racey across threads.  Any mild shuffling is enough. */
+static unsigned int global_rand_state;
+
 Sentence sentence_create(const char *input_string, Dictionary dict)
 {
 	Sentence sent;
@@ -770,6 +783,7 @@ Sentence sentence_create(const char *input_string, Dictionary dict)
 	sent->null_count = 0;
 	sent->parse_info = NULL;
 	sent->string_set = string_set_create();
+	sent->rand_state = global_rand_state;
 
 	sent->q_pruned_rules = FALSE;
 #ifdef USE_FAT_LINKAGES
@@ -896,6 +910,7 @@ void sentence_delete(Sentence sent)
 #ifdef USE_FAT_LINKAGES
 	if (sent->is_conjunction) xfree(sent->is_conjunction, sizeof(char)*sent->length);
 #endif /* USE_FAT_LINKAGES */
+	global_rand_state = sent->rand_state;
 	xfree((char *) sent, sizeof(struct Sentence_s));
 }
 
@@ -1134,7 +1149,7 @@ static void chart_parse(Sentence sent, Parse_Options opts)
 		}
 
 		/* Give up if the parse count is overflowing */
-		if (PARSE_NUM_OVERFLOW < total)
+		if ((PARSE_NUM_OVERFLOW < total) || (total < 0))
 		{
 			if (verbosity > 0)
 			{
@@ -1143,6 +1158,7 @@ static void chart_parse(Sentence sent, Parse_Options opts)
 					sent->null_count, total);
 			}
 			total = (total>INT_MAX) ? INT_MAX : total;
+			total = (total<0) ? INT_MAX : total;
 		}
 
 		sent->num_linkages_found = (int) total;
