@@ -18,7 +18,7 @@
 
 /* Disjunct utilities ... */
 void free_disjuncts(Disjunct *);
-int count_disjuncts(Disjunct *);
+unsigned int count_disjuncts(Disjunct *);
 Disjunct * copy_disjunct(Disjunct * );
 Disjunct * catenate_disjuncts(Disjunct *, Disjunct *);
 Disjunct * eliminate_duplicate_disjuncts(Disjunct * );
