@@ -30,9 +30,6 @@ typedef struct
 	Sentence      sent;
 	Linkage       linkage;
 	int           num_linkages, cur_linkage;
-#if DO_PHRASE_TREE
-	CNode*        tree;
-#endif
 } per_thread_data;
 
 /* XXX FIXME
@@ -72,6 +69,7 @@ static per_thread_data * get_ptd(JNIEnv *env, jclass cls)
 
 static void setup_panic_parse_options(Parse_Options opts)
 {
+	parse_options_set_repeatable_rand(opts, FALSE);
 	parse_options_set_disjunct_costf(opts, 3.0f);
 	parse_options_set_min_null_count(opts, 1);
 	parse_options_set_max_null_count(opts, MAX_SENTENCE);
@@ -88,10 +86,10 @@ static void setup_panic_parse_options(Parse_Options opts)
 static inline void test(void)
 {
 #ifdef DEBUG
-	printf("%d\n",word_contains("said",PAST_TENSE_FORM_MARKER,dict));
-	printf("%d\n",word_contains("gave.v",PAST_TENSE_FORM_MARKER,dict));
-	printf("%d\n",word_contains("have",PAST_TENSE_FORM_MARKER,dict));
-	printf("%d\n",word_contains("had",PAST_TENSE_FORM_MARKER,dict));
+	printf("%d\n", word_contains("said", PAST_TENSE_FORM_MARKER, dict));
+	printf("%d\n", word_contains("gave.v", PAST_TENSE_FORM_MARKER, dict));
+	printf("%d\n", word_contains("have", PAST_TENSE_FORM_MARKER, dict));
+	printf("%d\n", word_contains("had", PAST_TENSE_FORM_MARKER, dict));
 #endif
 }
 
@@ -135,6 +133,13 @@ static per_thread_data * init(JNIEnv *env, jclass cls)
 	memset(ptd, 0, sizeof(per_thread_data));
 
 	ptd->opts = parse_options_create();
+
+	/* Disable repeatable_rand. We'll be using Java primarily to
+	 * parse large texts, and we want a good distribution when
+	 * linkages overflow (very rare for mature grammars, but crucial
+	 * for grammar learning. That is, for the 'any' language.
+	 */
+	parse_options_set_repeatable_rand(ptd->opts, FALSE);
 	parse_options_set_disjunct_costf(ptd->opts, 2.0f);
 	parse_options_set_max_sentence_length(ptd->opts, MAX_SENTENCE-3);
 	parse_options_set_max_parse_time(ptd->opts, 30);
@@ -164,12 +169,6 @@ static void finish(per_thread_data *ptd)
 		sentence_delete(ptd->sent);
 	ptd->sent = NULL;
 
-#if DO_PHRASE_TREE
-	if (tree)
-		linkage_free_constituent_tree(tree);
-	tree = NULL;
-#endif
-
 	if (ptd->linkage)
 		linkage_delete(ptd->linkage);
 	ptd->linkage = NULL;
@@ -234,6 +233,11 @@ static void jParse(JNIEnv *env, per_thread_data *ptd, char* inputString)
 	Parse_Options opts = ptd->opts;
 	int jverbosity = parse_options_get_verbosity(opts);
 
+	/* Clean up a bit */
+	if (ptd->linkage)
+		linkage_delete(ptd->linkage);
+	ptd->linkage = NULL;
+
 	if (ptd->sent)
 		sentence_delete(ptd->sent);
 
@@ -297,7 +301,7 @@ static void jParse(JNIEnv *env, per_thread_data *ptd, char* inputString)
 		ptd->num_linkages = sentence_parse(ptd->sent, ptd->panic_parse_opts);
 		if (jverbosity > 0)
 		{
-			if (parse_options_timer_expired(ptd->panic_parse_opts)) 
+			if (parse_options_timer_expired(ptd->panic_parse_opts))
 				prt_error("Error: JNI: Panic timer is expired!\n");
 		}
 	}
@@ -310,7 +314,7 @@ static void makeLinkage(per_thread_data *ptd)
 		if (ptd->linkage)
 			linkage_delete(ptd->linkage);
 
-		ptd->linkage = linkage_create(ptd->cur_linkage,ptd->sent,ptd->opts);
+		ptd->linkage = linkage_create(ptd->cur_linkage, ptd->sent, ptd->opts);
 
 #ifdef USE_FAT_LINKAGES
 		if (!ptd->linkage) return;
@@ -318,13 +322,6 @@ static void makeLinkage(per_thread_data *ptd)
 		linkage_set_current_sublinkage(ptd->linkage,
 		                        linkage_get_num_sublinkages(ptd->linkage)-1);
 #endif /* USE_FAT_LINKAGES */
-
-#if DO_PHRASE_TREE
-		if (tree)
-			linkage_free_constituent_tree(tree);
-		tree = linkage_constituent_tree(linkage);
-		printTree(tree);
-#endif
 	}
 }
 
@@ -371,14 +368,14 @@ JNIEXPORT void JNICALL
 Java_org_linkgrammar_LinkGrammar_setDictionariesPath(JNIEnv *env,
                                               jclass cls, jstring path)
 {
-	const char *nativePath = (*env)->GetStringUTFChars(env,path, 0);
+	const char *nativePath = (*env)->GetStringUTFChars(env, path, 0);
 
 	// Java passes null pointers as the string "null"
 	if (nativePath && strcmp(nativePath, "null"))
 	{
 		dictionary_set_data_dir(nativePath);
 	}
-	(*env)->ReleaseStringUTFChars(env,path, nativePath);
+	(*env)->ReleaseStringUTFChars(env, path, nativePath);
 }
 
 /*
@@ -389,7 +386,7 @@ Java_org_linkgrammar_LinkGrammar_setDictionariesPath(JNIEnv *env,
 JNIEXPORT void JNICALL
 Java_org_linkgrammar_LinkGrammar_setLanguage(JNIEnv *env, jclass cls, jstring str)
 {
-	in_language = (*env)->GetStringUTFChars(env,str,0);
+	in_language = (*env)->GetStringUTFChars(env, str, 0);
 }
 
 /*
@@ -404,6 +401,13 @@ Java_org_linkgrammar_LinkGrammar_setMaxLinkages(JNIEnv *env, jclass cls, jint ma
 	parse_options_set_linkage_limit(ptd->opts, maxLinkages);
 }
 
+JNIEXPORT jint JNICALL
+Java_org_linkgrammar_LinkGrammar_getMaxLinkages(JNIEnv *env, jclass cls)
+{
+	per_thread_data *ptd = get_ptd(env, cls);
+	return parse_options_get_linkage_limit(ptd->opts);
+}
+
 /*
  * Class:      LinkGrammar
  * Method:     init
@@ -435,12 +439,12 @@ Java_org_linkgrammar_LinkGrammar_parse(JNIEnv *env, jclass cls, jstring str)
 {
 	const char *cStr;
 	char * tmp;
-	per_thread_data *ptd = get_ptd(env, cls);;
-	cStr = (*env)->GetStringUTFChars(env,str,0);
+	per_thread_data *ptd = get_ptd(env, cls);
+	cStr = (*env)->GetStringUTFChars(env, str, 0);
 	tmp = strdup(cStr);
 	jParse(env, ptd, tmp);
 	free(tmp);
-	(*env)->ReleaseStringUTFChars(env,str,cStr);
+	(*env)->ReleaseStringUTFChars(env, str, cStr);
 }
 
 /*
@@ -451,7 +455,7 @@ Java_org_linkgrammar_LinkGrammar_parse(JNIEnv *env, jclass cls, jstring str)
 JNIEXPORT void JNICALL
 Java_org_linkgrammar_LinkGrammar_close(JNIEnv *env, jclass cls)
 {
-	per_thread_data *ptd = get_ptd(env, cls);;
+	per_thread_data *ptd = get_ptd(env, cls);
 	finish(ptd);
 }
 
@@ -463,7 +467,7 @@ Java_org_linkgrammar_LinkGrammar_close(JNIEnv *env, jclass cls)
 JNIEXPORT jint JNICALL
 Java_org_linkgrammar_LinkGrammar_getNumWords(JNIEnv *env, jclass cls)
 {
-	per_thread_data *ptd = get_ptd(env, cls);;
+	per_thread_data *ptd = get_ptd(env, cls);
 	return linkage_get_num_words(ptd->linkage);
 }
 
