@@ -232,6 +232,12 @@ size_t lg_mbrtowc(wchar_t *pwc, const char *s, size_t n, mbstate_t *ps)
 }
 #endif /* defined(_MSC_VER) || defined(__MINGW32__) */
 
+#if __APPLE__
+/* Junk, to keep the Mac OSX linker happy, because this is listed in
+ * the link-grammar.def symbol export file.  */
+void lg_mbrtowc() {}
+#endif
+
 static int wctomb_check(char *s, wchar_t wc)
 {
 	int nr;
